package org.processmining.resourcerecommendation.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.resourcerecommendation.help.RecommendResourcesHelp;
import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.Dimension;
import org.processmining.resourcerecommendation.models.Request;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.impl.ResourceProcessFactory;
import org.processmining.resourcerecommendation.parameters.RecommendResourcesParameters;

public class RecommendResourcesDialog extends JPanel {

	private static final long serialVersionUID = 7568392196251751845L;

	private ResourceKnowledge knowledge;
	private RecommendResourcesParameters parameters;
	private int numPanels;
	
	public RecommendResourcesDialog(UIPluginContext context, ResourceKnowledge knowledge,
			final RecommendResourcesParameters parameters) {
		this.knowledge = knowledge;
		this.parameters = parameters;
		this.numPanels = 4;
	}
	
	public String[] getTitles(){
		String[] titles = { "Welcome", "Configure Algorithm", "Configure Requests", "Configure Weights" };
		return titles;
	}
	
	public int getNumPanels(){
		return numPanels;
	}
	
	public JPanel getPanel(int n) {
		removeAll();
		
		if (n == 0){
			double size[][] = { {30, TableLayoutConstants.FILL, 30},{30, TableLayoutConstants.FILL, 30}};
			setLayout(new TableLayout(size));
			JLabel welcome = new JLabel(RecommendResourcesHelp.PARAMETERS_DESCRIPTION);
			add(welcome, "1,1");
			return this;
			
		}
		if (n == 1){
			//Layout
			double cols[] = {30, 50, 600, 30};
			double rows[] = {30, 30,30,15,30, 30};
			double size[][] = {cols, rows};
			setLayout(new TableLayout(size));
			
			
			final JLabel explanation = new JLabel("");
			add(explanation, "2,2");
			if(parameters.getAlgorithm().equals(RecommendResourcesParameters.ALGORITHM.ALLOCATION)){
				explanation.setText(RecommendResourcesHelp.ALLOCATION_ALGORITHM_DESCRIPTION);
			}
			else if (parameters.getAlgorithm().equals(RecommendResourcesParameters.ALGORITHM.RECOMENDATION)){
				explanation.setText(RecommendResourcesHelp.RECOMMENDATION_ALGORITHM_DESCRIPTION);
			}
			
			SpinnerModel spinnerModel =
			         new SpinnerNumberModel(parameters.getNumRecommendations(), //initial value
			            1, //min
			            knowledge.getResources().size(), //max
			            1);//step
			final JSpinner spinner = new JSpinner(spinnerModel);
		      spinner.addChangeListener(new ChangeListener() {
		         public void stateChanged(ChangeEvent e) {
		        	 int num = (int) spinner.getModel().getValue();
		        	 parameters.setNumRecommendations(num);
		         }
		      });
		      add(spinner, "1,4"); 
		      final JLabel numLabel = new JLabel("Number of recommendations");
				add(numLabel, "2,4");
			
			final ProMComboBox<RecommendResourcesParameters.ALGORITHM> comboBox = 
					new ProMComboBox<RecommendResourcesParameters.ALGORITHM>(RecommendResourcesParameters.ALGORITHM.values());
			comboBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					RecommendResourcesParameters.ALGORITHM selected = (RecommendResourcesParameters.ALGORITHM) comboBox.getSelectedItem();
					parameters.setAlgorithm(selected);
					
					if(selected.equals(RecommendResourcesParameters.ALGORITHM.ALLOCATION)){
						explanation.setText(RecommendResourcesHelp.ALLOCATION_ALGORITHM_DESCRIPTION);
						parameters.setNumRecommendations(1);
						spinner.setEnabled(false);
						spinner.setValue(1);
					}
					else if (selected.equals(RecommendResourcesParameters.ALGORITHM.RECOMENDATION)){
						explanation.setText(RecommendResourcesHelp.RECOMMENDATION_ALGORITHM_DESCRIPTION);
						spinner.setEnabled(true);
					}
				}
			});
			add(comboBox, "2,1");
			
			
			

			
			return this;
			
		}
		
		else if (n == 2){
			//Layout
			final List<Characterization> characterizations = knowledge.getCharacterizations();
			
			double cols[] = {80,TableLayoutConstants.FILL};
			double rows[] = new double[characterizations.size()];
			for (int i = 0; i < characterizations.size(); i++){
				rows[i] = 30;
			}
			double size[][] = {cols,rows};
			setLayout(new TableLayout(size));
			
			final List<JSpinner> spinners = new ArrayList<JSpinner>();
			
			for (Characterization c: characterizations){
				SpinnerModel spinnerModel =
				         new SpinnerNumberModel(parameters.getNumRequests(c.getId()), //initial value
				            0, //min
				            10000, //max
				            1);//step
				JSpinner spinner = new JSpinner(spinnerModel);
			      spinner.addChangeListener(new ChangeListener() {
			         public void stateChanged(ChangeEvent e) {
			        	Map<String,Request> requestMap = new LinkedHashMap<String, Request>();
			            for(int i=0; i<spinners.size(); i++){
			            	JSpinner spinner = spinners.get(i);
			            	Characterization characterization = characterizations.get(i);
			            	int num = (int) spinner.getModel().getValue();
			            	for (int j= 0; j < num; j++){
			            		String uuid = UUID.randomUUID().toString();
			            		String requestId = "REQ:"+uuid;
			            		String requestDescription = "REQ:"+uuid;
			            		Request request = ResourceProcessFactory.createRequest(requestId, requestDescription, characterization.getId());
			            		requestMap.put(requestId, request);
			            	}
			            }
			            parameters.setRequests(requestMap);
			         }
			      });
			      spinners.add(spinner);
			}
			
			for (int i=0; i< characterizations.size(); i++){
				add(spinners.get(i), "0,"+i);
				add(new JLabel("\t"+characterizations.get(i).getId()+": "+characterizations.get(i).getDescription()), "1,"+i);
			}
			
			
		}
		else if (n == 3){
			
			
			//Layout
			final List<Dimension> dimensions = knowledge.getDimensions();
			
			double cols[] = {80,TableLayoutConstants.FILL};
			double rows[] = new double[dimensions.size()];
			for (int i = 0; i < dimensions.size(); i++){
				rows[i] = 30;
			}
			double size[][] = {cols,rows};
			setLayout(new TableLayout(size));
			
			final List<JSpinner> spinners = new ArrayList<JSpinner>();
			
			for (Dimension d: dimensions){
				double weight = parameters.getWeights().get(d.getId());
				SpinnerModel spinnerModel =
				         new SpinnerNumberModel(weight, //initial value
				            0.0, //min
				            1.0, //max
				            0.05);//step
				JSpinner spinner = new JSpinner(spinnerModel);
			      spinner.addChangeListener(new ChangeListener() {
			         public void stateChanged(ChangeEvent e) {
			        	Map<String,Double> weightMap = new LinkedHashMap<String, Double>();
			            for(int i=0; i<spinners.size(); i++){
			            	JSpinner spinner = spinners.get(i);
			            	Dimension dimension = dimensions.get(i);
			            	double num = (double) spinner.getModel().getValue();
			            	weightMap.put(dimension.getId(), num);
			            }
			            parameters.setWeights(weightMap);
			         }
			      });
			      spinners.add(spinner);
			}
			
			for (int i=0; i< dimensions.size(); i++){
				add(spinners.get(i), "0,"+i);
				add(new JLabel("\t"+dimensions.get(i).getId()), "1,"+i);
			}
			
		}

		return this;
	}

	public void finish() {
		System.out.println("Finished");
	}
}
