package org.processmining.resourcerecommendation.experiments;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;

import org.deckfour.xes.in.XesXmlParser;
import org.deckfour.xes.model.XLog;
import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.Dimension;
import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.RecommendationElement;
import org.processmining.resourcerecommendation.models.Request;
import org.processmining.resourcerecommendation.models.Resource;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.ResourceRecommendation;
import org.processmining.resourcerecommendation.models.impl.ResourceProcessFactory;
import org.processmining.resourcerecommendation.parameters.GenerateResourceKnowledgeParameters;
import org.processmining.resourcerecommendation.parameters.RecommendResourcesParameters;
import org.processmining.resourcerecommendation.plugins.GenerateResourceKnowledgePlugin;
import org.processmining.resourcerecommendation.plugins.RecommendResourcesPlugin;

import au.com.bytecode.opencsv.CSVWriter;

public class ExperimentsDKE {

	public static void main(String[] args) throws Exception {
		
		//SETTINGS
		//******************************************
		long idExp = (new Date()).getTime();
		String LOG_PATH = "/Users/jmunoz/Dropbox/code/ResourceRecommendation/src/org/processmining/resourcerecommendation/examples/logtesting1465248329216.xes";
		String RESULTS_FOLDER_PATH = "/Users/jmunoz/Dropbox/code/ResourceRecommendation/src/org/processmining/resourcerecommendation/examples/";
		String RESULTS_KNOWLEDGE_PATH = RESULTS_FOLDER_PATH+idExp+"_knowledge.csv";
		//******************************************
		
		// Instantiate a context for us to use
		//CLIContext uiContext = new CLIContext();
		//PluginContext context = new CLIPluginContext(uiContext, "Dummy Context");
		
		// Load log
		XesXmlParser parser = new XesXmlParser();
		XLog log = parser.parse(new File(LOG_PATH)).get(0);
		
		//Generate Knowledge
		GenerateResourceKnowledgePlugin pluginGRK = new GenerateResourceKnowledgePlugin();
		GenerateResourceKnowledgeParameters parameters = new GenerateResourceKnowledgeParameters(log);
		ResourceKnowledge knowledge = pluginGRK.apply(log, parameters);
		
		//Write Knowledge
		List<String[]> lines = new ArrayList<String[]>();
		for (Dimension dimension: knowledge.getDimensions()){
			String dimensionId = dimension.getId();
			for (Characterization characterization: knowledge.getCharacterizations()){
				String characterizationId = characterization.getId();
				for (Resource resource: knowledge.getResources()){
					String resourceId = resource.getId();
					double metric = knowledge.getMetric(dimensionId, characterizationId, resourceId);
					String[] line = {dimensionId,characterizationId,resourceId,String.valueOf(metric)};
					lines.add(line);
				}
			}
		}
		CSVWriter writer = new CSVWriter(new FileWriter(new File(RESULTS_KNOWLEDGE_PATH)));
		writer.writeAll(lines);
		writer.close();

		
		//Experiment configurations
		Map<String,RecommendResourcesParameters> configurations = buildConfigurations(knowledge);
		
		int exp = 0;
		for (Entry<String,RecommendResourcesParameters> entry: configurations.entrySet()){
			String configurationDescription = entry.getKey();
			RecommendResourcesParameters configuration = entry.getValue();
			
			RecommendResourcesPlugin pluginRR = new RecommendResourcesPlugin();
			ResourceRecommendation recommendation = pluginRR.apply(knowledge, configuration);
			
			//TODO Write in CSV similar to knowledge (write a different file 
			// Write everything: algorithm, weights, requests, and results
			
			//Meanwhile
			System.out.println();
			System.out.println("--EXPERIMENT "+ exp +"-------------------------------------");
			System.out.println("Description: "+configurationDescription);
			for (Entry<Request, List<RecommendationElement>> entryR: recommendation.getRankings().entrySet()){
				System.out.println(entryR.getKey().getCharacterizationId()+"==>");
				int i = 1;
				for (RecommendationElement element: entryR.getValue()){
					System.out.println("\t"+i+"->"+element.getResourceId()+"("+element.getOverallScore()+")");
					i ++;
				}
				
			}
			System.out.println("---------------------------------------------------");
			exp ++;
		}
		
			
	}
	
	private static Map<String,RecommendResourcesParameters> buildConfigurations(ResourceKnowledge knowledge){
		Map<String,RecommendResourcesParameters> configurations = new LinkedHashMap<String, RecommendResourcesParameters>();
		
		//CONF1
		RecommendResourcesParameters cf01 = new RecommendResourcesParameters(knowledge);
		//1-Algorithm
		cf01.setAlgorithm(RecommendResourcesParameters.ALGORITHM.RECOMENDATION);
		//2- Recommendations
		cf01.setNumRecommendations(3);
		//3- Weights
		for (Dimension dimension: knowledge.getDimensions()){
			String dimensionId = dimension.getId();
			if (dimensionId == Dimensions.FREQUENCY){
				cf01.setWeight(dimensionId, 1.0);
			}
			else{
				cf01.setWeight(dimensionId, 0.0);
			}
		}
		//4- Requests
		Map<String,Request> requests01 = new LinkedHashMap<String,Request>();
		String uuid01 = UUID.randomUUID().toString();
		String characterizationIdc00000 = "c00000";
		Request request01 = ResourceProcessFactory.createRequest("REQ:"+uuid01, "REQ:"+uuid01, characterizationIdc00000);
		requests01.put("REQ:"+uuid01, request01);
		cf01.setRequests(requests01);
		//5-Include it
		configurations.put("rec3 only frequency", cf01);
		
		
		
		//CONF2
		RecommendResourcesParameters cf02 = new RecommendResourcesParameters(knowledge);
		//1-Algorithm
		cf02.setAlgorithm(RecommendResourcesParameters.ALGORITHM.RECOMENDATION);
		//2- Recommendations
		cf02.setNumRecommendations(3);
		//3- Weights
		for (Dimension dimension: knowledge.getDimensions()){
			String dimensionId = dimension.getId();
			if (dimensionId == Dimensions.WORKLOAD){
				cf02.setWeight(dimensionId, 1.0);
			}
			else{
				cf02.setWeight(dimensionId, 0.0);
			}
		}
		//4- Requests
		Map<String,Request> requests02 = new LinkedHashMap<String,Request>();
		String uuid02 = UUID.randomUUID().toString();
		String characterizationIdc00001 = "c00001";
		Request request02 = ResourceProcessFactory.createRequest("REQ:"+uuid02, "REQ:"+uuid02, characterizationIdc00001);
		requests02.put("REQ:"+uuid01, request01);
		cf02.setRequests(requests02);
		//5-Include it
		configurations.put("rec3 only workload", cf02);
		
		
		
		//TODO All the configurations that we want
		
		//RETURN
		return configurations;
	}

}
