// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmining.resourcerecommendation.experiments;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.framework.util.Pair;
import org.processmining.resourcerecommendation.utils.resrecxes.XResRecExtension;

public class GenerateLog01 {

	public static void main(String[] args) throws IOException {
		
		//Paths and names of the output
		String PATH = "/Users/jmunoz/Dropbox/benchmarks/resourcerecommendation/";
		String NAME = "log"+ (new Date()).getTime() +".xes";
		
		//Names of traces, resources, events, ...
		String TRACE_NAME = "t";
		
		//Factory to create the log elements
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		
		//Create log
		XLog log = factory.createLog();
		
		//Add the required extensions
		log.getExtensions().add(XConceptExtension.instance());
		log.getExtensions().add(XLifecycleExtension.instance());
		log.getExtensions().add(XTimeExtension.instance());
		log.getExtensions().add(XOrganizationalExtension.instance());
		log.getExtensions().add(XResRecExtension.instance());
		
		
		//Add log attributes: Typical	
		XConceptExtension.instance().assignName(log, NAME);
			//Use the typical: start, complete, ... lifecycle model
		XLifecycleExtension.instance().assignModel(log, XLifecycleExtension.VALUE_MODEL_STANDARD);
		
		
		//Add log attributes: ResRec: Resources
		Map<String, String> resourceMap = new LinkedHashMap<String,String>();
		List<String> resourceIds = new ArrayList<String>();
		resourceMap.put("r00000", "Alice");
		resourceIds.add("r00000");
		resourceMap.put("r00001", "Bob");
		resourceIds.add("r00001");
		resourceMap.put("r00002", "Charlie");
		resourceIds.add("r00002");
		resourceMap.put("r00003", "David");
		resourceIds.add("r00003");
		resourceMap.put("r00004", "Eva");
		resourceIds.add("r00004");
		XResRecExtension.instance().assignResources(log, resourceMap);
		
		//Add log attributes: ResRec: Units
		Map<String, String> unitsMap = new LinkedHashMap<String,String>();
		unitsMap.put("u00000", "Level 1");
		unitsMap.put("u00001", "Level 2");
		XResRecExtension.instance().assignUnits(log, unitsMap);
		
		
		//Add log attributes: ResRec: Factors
		Map<Pair<String, String>,Map<String, String>> factorsMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		
		Map<String,String> mapf0 = new LinkedHashMap<String,String>();
		mapf0.put("p00000", "Laser Printer");
		mapf0.put("p00001", "Deployment Server");
		factorsMap.put(new Pair<String,String>("f00000", "Product"),mapf0);
		
		Map<String,String> mapf1 = new LinkedHashMap<String,String>();
		mapf1.put("l00000", "English");
		mapf1.put("l00001", "Spanish");
		factorsMap.put(new Pair<String,String>("f00001", "Language"),mapf1);
		
		XResRecExtension.instance().assignFactors(log, factorsMap);
		
		

		//Add log attributes: ResRec: Typologies
		Map<Pair<String, String>,Map<String, String>> typologiesMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		
		Map<String,String> mapt0 = new LinkedHashMap<String,String>();
		mapt0.put("f00000", "p00000");
		mapt0.put("f00001", "l00000");
		typologiesMap.put(new Pair<String,String>("t00000", "Printer-English"),mapt0);
		
		Map<String,String> mapt1 = new LinkedHashMap<String,String>();
		mapt1.put("f00000", "p00000");
		mapt1.put("f00001", "l00001");
		typologiesMap.put(new Pair<String,String>("t00001", "Printer-Spanish"),mapt1);
		
		Map<String,String> mapt2 = new LinkedHashMap<String,String>();
		mapt2.put("f00000", "p00001");
		mapt2.put("f00001", "l00001");
		typologiesMap.put(new Pair<String,String>("t00002", "Server-Spanish"),mapt2);
		
		XResRecExtension.instance().assignTypologies(log, typologiesMap);
		
		
		
		
		//Add log attributes: ResRec: Characterizations
		Map<Pair<String, String>,Pair<String, String>> characterizationMap = new LinkedHashMap<Pair<String, String>,Pair<String, String>>();
		List<String> characterizationIds = new ArrayList<String>();
			//Id, Description, Unit, Typology
		characterizationMap.put(new Pair<String, String>("c00000","L1-Printer-ESP"), new Pair<String, String>("u00000","t00001"));
		characterizationIds.add("c00000");
		characterizationMap.put(new Pair<String, String>("c00001","L2-Printer-ESP"), new Pair<String, String>("u00001","t00001"));
		characterizationIds.add("c00001");
		characterizationMap.put(new Pair<String, String>("c00002","L1-Server-ESP"), new Pair<String, String>("u00000","t00002"));
		characterizationIds.add("c00002");
		characterizationMap.put(new Pair<String, String>("c00003","L2-Server-ESP"), new Pair<String, String>("u00001","t00002"));
		characterizationIds.add("c00003");
		characterizationMap.put(new Pair<String, String>("c00004","L2-Printer-ENG"), new Pair<String, String>("u00001","t00000"));
		characterizationIds.add("c00004");
		XResRecExtension.instance().assignCharacterizations(log, characterizationMap);
		
			
		
		
		//Create traces of 1 Level only
		Random random = new Random();
		int numTraces = 100;
		for (int i=0; i < numTraces; i++){
			XTrace trace = factory.createTrace();
			XConceptExtension.instance().assignName(trace, TRACE_NAME+log.size());
			
			//Only two events for level
				//Random characterization and resource
			String cha = characterizationIds.get(random.nextInt(characterizationIds.size()));
			String res = resourceIds.get(random.nextInt(resourceIds.size()));
			
				//Level: start
			XEvent eventStart = factory.createEvent();
			XConceptExtension.instance().assignName(eventStart, cha);
			XConceptExtension.instance().assignInstance(eventStart, cha);
			XLifecycleExtension.instance().assignStandardTransition(eventStart, StandardModel.START);
			XOrganizationalExtension.instance().assignResource(eventStart, res);
			XTimeExtension.instance().assignTimestamp(eventStart, new Date());
			trace.add(eventStart);
			
				//Level: complete
			XEvent eventComplete = factory.createEvent();
			XConceptExtension.instance().assignName(eventComplete, cha);
			XConceptExtension.instance().assignInstance(eventComplete, cha);
			XLifecycleExtension.instance().assignStandardTransition(eventComplete, StandardModel.COMPLETE);
			XOrganizationalExtension.instance().assignResource(eventComplete, res);
			XTimeExtension.instance().assignTimestamp(eventComplete, new Date());
			XAttributeDiscrete quality = factory.createAttributeDiscrete("quality", 1, null);
			eventComplete.getAttributes().put("quality", quality);
			trace.add(eventComplete);
			
			
			log.add(trace);
			
		}
		
		//Save the log in a file
		OutputStream out = new FileOutputStream(PATH+NAME);
		//XesXmlGZIPSerializer serializer = new XesXmlGZIPSerializer(); //Compressed
		XesXmlSerializer serializer = new XesXmlSerializer(); //Not compressed
		serializer.serialize(log, out);
		out.close();
		

	}

}
