//By Mike

package org.processmining.resourcerecommendation.experiments;

import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XCostExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.framework.util.Pair;
import org.processmining.resourcerecommendation.utils.resrecxes.XResRecExtension;

public class GenerateLog03 {

	public static void main(String[] args) throws IOException, ParseException {
		
		//Paths and names of the output
		String PATH = "/Users/Mike/Desktop/";
		String NAME = "logsimulated"+ (new Date()).getTime() +".xes";
		
		//Add log attributes: ResRec: Resources		
		Map<String, String> resourceMap = new LinkedHashMap<String,String>();
		List<String> resourceIds = new ArrayList<String>();
		
		//Add log attributes: ResRec: Typologies
		Map<Pair<String, String>,Map<String, String>> typologiesMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		Map<String,String> maptopologykey = new LinkedHashMap<String,String>();
		
		//Add log attributes: ResRec: Factors
		Map<Pair<String, String>,Map<String, String>> factorsMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		Map<String,String> mapf0 = new LinkedHashMap<String,String>();
		Map<String,String> mapf1 = new LinkedHashMap<String,String>();
		
		
		//Names of traces, resources, events, ...
		String TRACE_NAME = "t";
				
		//Factory to create the log elements
		XFactory factory = XFactoryRegistry.instance().currentDefault();


		//Create log
		XLog log = factory.createLog();
		//Add the required extensions
				log.getExtensions().add(XConceptExtension.instance());
				log.getExtensions().add(XLifecycleExtension.instance());
				log.getExtensions().add(XTimeExtension.instance());
				log.getExtensions().add(XOrganizationalExtension.instance());
				log.getExtensions().add(XResRecExtension.instance());
				log.getExtensions().add(XCostExtension.instance());
				
				
		//Leer CSV
		Scanner MyFile = new Scanner(new FileReader("/Users/Mike/Desktop/DataSetEventLogTest.csv"));
		
		Integer i = 0;
		Integer j = 0;
		Integer z = 0;
		Integer indextop = 0;
		String line="";
		MyFile.nextLine();
		while (MyFile.hasNextLine())
		{
			line = MyFile.nextLine();
			String[] splitline= line.split(";");
			if (!(mapf0.containsKey(splitline[1]))){ // Factor: SP1, SP2
				mapf0.put(splitline[1],"s0000"+ String.valueOf(i++));
				//i++;
			}
			if (!(mapf1.containsKey(splitline[2]))){ //Factor: Product line
				mapf1.put(splitline[2], "p0000" + String.valueOf(j++));
				//j++;
			}
			
			if (!(maptopologykey.containsKey(splitline[1] + " "+ splitline[2]))){
				maptopologykey.put(splitline[1] + " "+ splitline[2], "t0000"+ String.valueOf(indextop));

				Map<String, String> m=new HashMap<String, String>();
				m.put("f00000", mapf0.get(splitline[1]));
				m.put("f00001", mapf1.get(splitline[2]));
				typologiesMap.put(new Pair<String, String>(splitline[1] + "-"+ splitline[2],"t0000"+ String.valueOf(indextop)),m);				
				
				indextop++;
			}

			if (!(resourceMap.containsKey(splitline[3]))){ //Resources
				resourceMap.put(splitline[3],splitline[3]);
			}
			
			factorsMap.put(new Pair<String,String>("f00000", "Subprocess"),mapf0);
			factorsMap.put(new Pair<String,String>("f00001", "Productline"),mapf1);
			
		}
		
		XResRecExtension.instance().assignResources(log, resourceMap);
		XResRecExtension.instance().assignFactors(log, factorsMap);
		XResRecExtension.instance().assignTypologies(log, typologiesMap);
		
		Map<String, String> unitsMap = new LinkedHashMap<String,String>();
		unitsMap.put("u00000", "Level");
		XResRecExtension.instance().assignUnits(log, unitsMap);
		
		
		//Add log attributes: ResRec: Characterizations
		Map<Pair<String, String>,Pair<String, String>> characterizationMap = new LinkedHashMap<Pair<String, String>,Pair<String, String>>();
		List<String> characterizationIds = new ArrayList<String>();
		
		for (Map.Entry<String, String> unit: unitsMap.entrySet()){
			for (Map.Entry<String, String> f0: maptopologykey.entrySet()){
					characterizationMap.put(new Pair<String, String>("c0000"+String.valueOf(z++), unit.getValue()+ " " + f0.getKey()), new Pair<String, String>(unit.getKey(),f0.getValue()));
				}
		}
		
		//Id, Description, Unit, Typology (DMSOne, Problem)
		XResRecExtension.instance().assignCharacterizations(log, characterizationMap);
		
		//Read CSV
		Scanner MyFile2 = new Scanner(new FileReader("/Users/Mike/Desktop/DataSetEventLogTest.csv"));
		line = MyFile2.nextLine();
		
		//Create traces of 1 Level only
		while (MyFile2.hasNextLine()){
			XTrace trace = factory.createTrace();
			XConceptExtension.instance().assignName(trace, TRACE_NAME+log.size());
			
			//Only two events for level
			String cha = null;
			line = MyFile2.nextLine();
			String[] splitline= line.split(";");
			
			for (Map.Entry<Pair<String, String>,Pair<String, String>> characteristics: characterizationMap.entrySet()){
				Pair<String, String> l = characteristics.getKey();
				String[] e = l.getSecond().split(" ");
				if (splitline[2].equals(e[2])&& (splitline[1].equals(e[1]))){
					Pair<String, String> val = characteristics.getKey();
					cha=val.getFirst();
				}
			
			}
			
			SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy HH:mm");
			Date f_date = formatter.parse(splitline[7]);
					
			//Level: start
			XEvent eventStart = factory.createEvent();
			XConceptExtension.instance().assignName(eventStart, cha);
			XConceptExtension.instance().assignInstance(eventStart, cha);
			XLifecycleExtension.instance().assignStandardTransition(eventStart, StandardModel.START);
			XOrganizationalExtension.instance().assignResource(eventStart, splitline[3]);
			XTimeExtension.instance().assignTimestamp(eventStart, f_date);
			trace.add(eventStart);
			
			//Level: complete
			f_date = formatter.parse(splitline[8]);
			
			XEvent eventComplete = factory.createEvent();
			XConceptExtension.instance().assignName(eventComplete, cha);
			XConceptExtension.instance().assignInstance(eventComplete, cha);
			XLifecycleExtension.instance().assignStandardTransition(eventComplete, StandardModel.COMPLETE);
			XOrganizationalExtension.instance().assignResource(eventComplete, splitline[3]);
			XTimeExtension.instance().assignTimestamp(eventComplete, f_date);		
			XAttribute duration = factory.createAttributeContinuous("duration", Double.parseDouble(splitline[4]), null);
			eventComplete.getAttributes().put("duration", duration);
			XAttribute cost = factory.createAttributeContinuous("cost:total", Double.parseDouble(splitline[5]), null);
			eventComplete.getAttributes().put("cost:total", cost);
			XAttributeDiscrete quality = factory.createAttributeDiscrete("quality", Integer.parseInt(splitline[6]), null);
			eventComplete.getAttributes().put("quality", quality);
			trace.add(eventComplete);
			
			log.add(trace);
		}
		
		//Save the log in a file
		OutputStream out = new FileOutputStream(PATH+NAME);
		//XesXmlGZIPSerializer serializer = new XesXmlGZIPSerializer(); //Compressed
		XesXmlSerializer serializer = new XesXmlSerializer(); //Not compressed
		serializer.serialize(log, out);
		MyFile.close();
		MyFile2.close();
	}
}
