package org.processmining.resourcerecommendation.experiments;

import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Scanner;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XCostExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.framework.util.Pair;
import org.processmining.resourcerecommendation.utils.resrecxes.XResRecExtension;

public class GenerateLog05 {

	public static void main(String[] args) throws IOException, ParseException {
		
		//Paths and names of the output
		String PATH = "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\";
		String NAME = "logexperiment"+ (new Date()).getTime() +".xes";
		String PATH_Resources = "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\Resources.csv";
		String PATH_Workload =  "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\Workload.csv";
		String PATH_Log =  "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\DataSetEventLogTest.csv";
		String PATH_ExpertiseCatalog = "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\ExpertiseCatalog.csv";
		String PATH_RequiredExpertise = "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\Required_expertise.csv";
		String PATH_ResourceExpertise = "C:\\Users\\Mike\\Dropbox\\resourcerecommendation\\Resource_expertise.csv";
		
		//Random to create random values
		Random random = new Random(0);
		
		//Names of traces, resources, events, ...
		String TRACE_NAME = "t";
		
		//Factory to create the log elements
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		
		//Create log
		XLog log = factory.createLog();
		
		//Add the required extensions
		log.getExtensions().add(XConceptExtension.instance());
		log.getExtensions().add(XLifecycleExtension.instance());
		log.getExtensions().add(XTimeExtension.instance());
		log.getExtensions().add(XOrganizationalExtension.instance());
		log.getExtensions().add(XCostExtension.instance());
		log.getExtensions().add(XResRecExtension.instance());
		
		//Add log attributes: Typical	
		XConceptExtension.instance().assignName(log, NAME);
			//Use the typical: start, complete, ... lifecycle model
		XLifecycleExtension.instance().assignModel(log, XLifecycleExtension.VALUE_MODEL_STANDARD);
		
		
		//Add log attributes: ResRec: Resources
		//Leer CSV
		Scanner MyFile = new Scanner(new FileReader(PATH_Resources));
		Map<String, String> resourceMap = new LinkedHashMap<String,String>();
		List<String> resourceIds = new ArrayList<String>();
		String line="";
		MyFile.nextLine();
		while (MyFile.hasNextLine())
		{
			line = MyFile.nextLine();
			String[] splitline= line.split(";");
			if (!(resourceMap.containsKey(splitline[0]))){ 
				resourceMap.put(splitline[0],splitline[1]); // ResourceId, Name
				resourceIds.add(splitline[0]);
			}
		}
		XResRecExtension.instance().assignResources(log, resourceMap);
		
		//Add log attributes: ResRec: Workload
		//Leer CSV
		Scanner MyFile1 = new Scanner(new FileReader(PATH_Workload));
		Map<Pair<String,String>,Map<String,Long>> workloadMap = new LinkedHashMap<Pair<String,String>,Map<String,Long>>();
		line="";
		MyFile1.nextLine();
		while (MyFile1.hasNext())
		{
			line = MyFile1.nextLine();
			String[] splitline= line.split(";");
			String resourceId = splitline[0]; //resourceId
			Pair<String,String> keys = new Pair<String,String>(resourceId,"Workload of "+resourceId);
			Map<String,Long> values = new LinkedHashMap<String, Long>();
			Long max = Long.parseLong(splitline[1]);//max
			Long current = Long.parseLong(splitline[2]); //current
			values.put(XResRecExtension.KEY_WORKLOAD_MAX, max);
			values.put(XResRecExtension.KEY_WORKLOAD_CURRENT, current);
			workloadMap.put(keys, values);
		}
		XResRecExtension.instance().assignWorkloads(log, workloadMap);
		
		//Add log attributes: ResRec: Units
		//Map<String, String> unitsMap = new LinkedHashMap<String,String>();
		//unitsMap.put("u00000", "Level 1");
		//XResRecExtension.instance().assignUnits(log, unitsMap);		
		
		//Add log attributes: ResRec: Units, ResRec: Factors
		//Leer CSV
		Scanner MyFile2 = new Scanner(new FileReader(PATH_Log));
		Map<Pair<String, String>,Map<String, String>> factorsMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		//Map<String, String> unitsMap = new LinkedHashMap<String,String>();
		HashMap<String, String> unitsMap = new HashMap<String, String>();
		//Map<String,String> mapf0 = new LinkedHashMap<String,String>();
		//HashMap<String,String> mapf0 = new HashMap<String,String>();
		//Map<String,String> mapf1 = new LinkedHashMap<String,String>();
		HashMap<String,String> mapf1 = new HashMap<String,String>();
		
		Integer x = 0;
		Integer j = 0;
		Integer z = 0;
		Integer indextop = 0;
		line="";
		MyFile2.nextLine();
		while (MyFile2.hasNextLine())
		{							   //Add log attributes: ResRec: Units
			line = MyFile2.nextLine();
			String[] splitline= line.split(";");
			if (!(unitsMap.containsValue(splitline[1]))){ // Factor: Level1, Level2, etc.
				unitsMap.put("u0000"+ String.valueOf(x++),splitline[1]);
			}
		
			if (!(mapf1.containsValue(splitline[2]))){ //Factor: Product line
				mapf1.put("p0000" + String.valueOf(j++), splitline[2]);
			}
		}
		factorsMap.put(new Pair<String,String>("f0000"+ String.valueOf(indextop), "Product"),mapf1);
		
		XResRecExtension.instance().assignUnits(log, unitsMap);
		XResRecExtension.instance().assignFactors(log, factorsMap);
		
		//Add log attributes: ResRec: Typologies
		Map<Pair<String, String>,Map<String, String>> typologiesMap = new LinkedHashMap<Pair<String, String>,Map<String, String>>();
		Map<String,String> mapt0 = new LinkedHashMap<String,String>();
		Map<String,String> maptopologykey = new LinkedHashMap<String,String>();
		
		mapt0.put("f00000", "p00000");
		typologiesMap.put(new Pair<String,String>("t00000", "Printers"),mapt0);
		
		Map<String,String> mapt1 = new LinkedHashMap<String,String>();
		mapt1.put("f00000", "p00001");
		typologiesMap.put(new Pair<String,String>("t00001", "Servers"),mapt1);
		
		maptopologykey.put("t00000", "Printers");
		maptopologykey.put("t00001", "Servers");
		
		XResRecExtension.instance().assignTypologies(log, typologiesMap);
		
		//Add log attributes: ResRec: Characterizations
		Map<Pair<String, String>,Pair<String, String>> characterizationMap = new LinkedHashMap<Pair<String, String>,Pair<String, String>>();
		List<String> characterizationIds = new ArrayList<String>();
		//Id, Description, Unit, Typology
		for (Map.Entry<String, String> unit: unitsMap.entrySet()){
			for (Entry<String, String> f0: maptopologykey.entrySet()){
					characterizationMap.put(new Pair<String, String>("c0000"+String.valueOf(z), unit.getValue()+ "-" + f0.getValue()), new Pair<String, String>(unit.getKey(),f0.getKey()));
					characterizationIds.add("c0000"+String.valueOf(z++));
				}
		}
		XResRecExtension.instance().assignCharacterizations(log, characterizationMap);
		
		//Add log attributes: ResRec: Expertises
		//Leer CSV
		Scanner MyFile3 = new Scanner(new FileReader(PATH_ExpertiseCatalog));
		Map<String, String> expertisesMap = new LinkedHashMap<String,String>();
		List<String> expertiseIds = new ArrayList<String>();
		line="";
		MyFile3.nextLine();
		while (MyFile3.hasNextLine())
		{
			line = MyFile3.nextLine();
			String[] splitline= line.split(";");
			if (!(resourceMap.containsKey(splitline[0]))){ 
				expertisesMap.put(splitline[0],splitline[1]); // ExpertiseId, Description
				expertiseIds.add(splitline[0]);
			}
		}
		XResRecExtension.instance().assignExpertises(log, expertisesMap);
				
		//Add log attributes: ResRec: Resource Expertises
		//Leer CSV
		Scanner MyFile4 = new Scanner(new FileReader(PATH_ResourceExpertise));
		Map<Pair<String,String>,Map<String, Long>> expertiseResourcesMap = new LinkedHashMap<Pair<String,String>,Map<String, Long>>();
		line="";
		int count=1;//count expertise
		MyFile4.nextLine();
		while (MyFile4.hasNext()){
			for (String resource : resourceIds){
				Pair<String,String> pair = new Pair<String,String>(resource,"Expertises of "+resource);
				Map<String,Long> values = new LinkedHashMap<String,Long>();
				line = MyFile4.nextLine();
				String[] splitline= line.split(";");
				for (String expertise: expertiseIds){
					values.put(expertise, Long.parseLong(splitline[count]));
					count++;
				}
				expertiseResourcesMap.put(pair, values);
				count=1;
			}
		}
		XResRecExtension.instance().assignExpertiseResources(log, expertiseResourcesMap);
		
		
		//Add log attributes: ResRec: Expertises Required
		//Leer CSV
		Scanner MyFile5 = new Scanner(new FileReader(PATH_RequiredExpertise));
		Map<Pair<String,String>,Map<String, Long>> expertiseRequiredMap = new LinkedHashMap<Pair<String,String>,Map<String, Long>>();
		line="";
		count=1;//count expertise
		MyFile5.nextLine();
		while (MyFile5.hasNext()){
			for (String characterization : characterizationIds){
				Pair<String,String> pair = new Pair<String,String>(characterization,"Required for "+characterization);
				Map<String,Long> values = new LinkedHashMap<String,Long>();
				line = MyFile5.nextLine();
				String[] splitline= line.split(";");
				for (String expertise: expertiseIds){
					values.put(expertise, Long.parseLong(splitline[count]));
				}
				expertiseRequiredMap.put(pair, values);
			}
		}
		XResRecExtension.instance().assignExpertiseRequired(log, expertiseRequiredMap);
		
		
		//Create traces of 1 Level only
		//Read CSV
		Scanner MyFile6 = new Scanner(new FileReader(PATH_Log));
		line = MyFile6.nextLine();
		
		while (MyFile6.hasNextLine())
			try {
				{
					XTrace trace = factory.createTrace();
					XConceptExtension.instance().assignName(trace, TRACE_NAME+log.size());
					
					//Only two events for level
					String cha = null;
					line = MyFile6.nextLine();
					String[] splitline= line.split(";");
					
					for (Map.Entry<Pair<String, String>,Pair<String, String>> characteristics: characterizationMap.entrySet()){
						Pair<String, String> l = characteristics.getKey();
						String[] e = l.getSecond().split(" ");
						String compare = splitline[1]+"-"+splitline[2]; 
						if (compare.equals(e[0])){
						//if (splitline[2].equals(e[2])&& (splitline[1].equals(e[1]))){
							Pair<String, String> val = characteristics.getKey();
							cha=val.getFirst();
						}
					}
					
					SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy HH:mm");
					Date f_date = formatter.parse(splitline[7]);
					//Level: complete
					f_date = formatter.parse(splitline[8]);
					
					XEvent eventComplete = factory.createEvent();
					XConceptExtension.instance().assignName(eventComplete, cha);
					XConceptExtension.instance().assignInstance(eventComplete, cha);
					XLifecycleExtension.instance().assignStandardTransition(eventComplete, StandardModel.COMPLETE);
					XOrganizationalExtension.instance().assignResource(eventComplete, splitline[3]);
					XTimeExtension.instance().assignTimestamp(eventComplete, f_date);		
					XAttribute duration = factory.createAttributeContinuous("duration", Double.parseDouble(splitline[4]), null);
					eventComplete.getAttributes().put("duration", duration);
					XAttribute cost = factory.createAttributeContinuous("cost:total", Double.parseDouble(splitline[5]), null);
					eventComplete.getAttributes().put("cost:total", cost);
					XAttributeDiscrete quality = factory.createAttributeDiscrete("quality", Integer.parseInt(splitline[6]), null);
					eventComplete.getAttributes().put("quality", quality);
					trace.add(eventComplete);
					
					log.add(trace);
					
				}
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}//while
			
		
		//Save the log in a file
		OutputStream out = new FileOutputStream(PATH+NAME);
		XesXmlSerializer serializer = new XesXmlSerializer(); //Not compressed
		serializer.serialize(log, out);
		MyFile.close();
		MyFile1.close();
		MyFile2.close();
		MyFile3.close();
		MyFile4.close();
		MyFile5.close();
		MyFile6.close();
	}

}
