// By Mike
package org.processmining.resourcerecommendation.models.impl;

import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.CostCharacterization;
import org.processmining.resourcerecommendation.models.CostResource;

public class CostCharacterizationImpl implements CostCharacterization {
	
	private String characterizationId;
	private Map<String, CostResource> resources;
	
	protected CostCharacterizationImpl(String characterizationId){
		this.characterizationId = characterizationId;
		this.resources = new LinkedHashMap<String, CostResource>();
	}
	
	public void addCost(String resourceId, double cost){
		CostResource resource = resources.get(resourceId);
		if (resource == null){
			resource = ResourceProcessFactory.createCostResource(resourceId);
			resources.put(resourceId, resource);
		}
		resource.addCost(cost);
	}
	
	public long getTotal(String resourceId) {
		long result = resources.containsKey(resourceId) ? 
				resources.get(resourceId).getTotal() : 0;
		return result;	
	}
	
	public long getTotal(){
		long result = 0;
		for (CostResource resource: resources.values()){
			result += resource.getTotal();
		}
		return result;
	}

	public String getCharacterizationId() {
		return characterizationId;
	}

	public Double getMin() {
		Double min = Double.MAX_VALUE;
		for (CostResource resource: resources.values()){
			if (resource.getTotal()> 0 && resource.getMin()< min) 
				min= resource.getMin(); 
		}
		return min;
	}
	
	public Double getMax() {
		Double max = Double.MIN_VALUE;
		for (CostResource resource: resources.values()){
			if (resource.getTotal()> 0 && resource.getMax()> max) 
				max= resource.getMax(); 
		}
		return max;
	}

	public Double getAverage(String resourceId) {
		Double result = resources.containsKey(resourceId) ? 
				resources.get(resourceId).getAverage() : 0.0;
		return result;
	}
}
