// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.FrequencyCharacterization;
import org.processmining.resourcerecommendation.models.FrequencyDimension;

public class FrequencyDimensionImpl implements FrequencyDimension {
	
	private String id;
	private Map<String, FrequencyCharacterization> characterizations;
	
	protected FrequencyDimensionImpl(){
		this.id = Dimensions.FREQUENCY;
		this.characterizations = new LinkedHashMap<String, FrequencyCharacterization>();
	}
	public String getId(){
		return id;
	}
	
	public void addFrequency(String characterizationId, String resourceId){
		FrequencyCharacterization characterization = characterizations.get(characterizationId);
		if (characterization == null){
			characterization = ResourceProcessFactory.createFrequencyCharacterization(characterizationId);
			this.characterizations.put(characterizationId, characterization);
		}
		characterization.addFrequency(resourceId);	
	}
	
	public double getMetric(String characterizationId, String resourceId){
		if (!characterizations.containsKey(characterizationId)) return 0.0;
		long totalResource = characterizations.get(characterizationId).getTotal(resourceId);
		long totalAll = characterizations.get(characterizationId).getTotal();
		if(totalResource == 0) return 0.0;
		double result = ((Math.log(totalResource)) + 1) / ((Math.log(totalAll)) +1);
		//System.out.println(resourceId + " " + result);
		return result;
	}

	public ArrayList<Double> getMetric(String[] characterizationIds, String[] resourceIds) {
		ArrayList<Double> finalResult = new ArrayList<Double>();
		for (int i=0; i<resourceIds.length; i++){
			finalResult.add(getMetric(characterizationIds[i], resourceIds[i]));
		}
		return finalResult;
	}


}
