// By Mike
package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.OverQualificationDimension;

public class OverQualificationDimensionImpl implements OverQualificationDimension {
	
	private String id;
	private Map<String, Map<String,Long>> characterizations;
	private Map<String, Map<String,Long>> resources;
	private long EX_BOTTOM = 0;
	private long EX_TOP = 5;
	
	protected OverQualificationDimensionImpl(){
		this.id = Dimensions.OVERQUALIFICATION;
		this.characterizations = new LinkedHashMap<String, Map<String,Long>>();
		this.resources = new LinkedHashMap<String, Map<String,Long>>();
	}
	
	public String getId(){
		return id;
	}
	
	public void addRequiredExpertise(String characterizationId, String expertiseId, Long value) {
		Map<String,Long> expertises = characterizations.get(characterizationId);
		if (expertises == null){
			expertises = new LinkedHashMap<String,Long>();
			characterizations.put(characterizationId, expertises);
		}
		expertises.put(expertiseId, value);
	}

	public void addResourceExpertise(String resourceId, String expertiseId, Long value) {
		Map<String,Long> expertises = resources.get(resourceId);
		if (expertises == null){
			expertises = new LinkedHashMap<String,Long>();
			resources.put(resourceId, expertises);
		}
		expertises.put(expertiseId, value);
		
	}
	
	public double getMetric(String characterizationId, String resourceId){
		if (!characterizations.containsKey(characterizationId)) return 0.0; // No requirements, 0
		if (!resources.containsKey(resourceId)) return 0.0; //No resource expertise, 0
		Map<String,Long> cMap = this.characterizations.get(characterizationId);
		Map<String,Long> rMap = this.resources.get(resourceId);
		double sum = 0.0;
		double num = 0.0;
		for (Entry<String,Long> cExp : cMap.entrySet()){
			String exId = cExp.getKey();
			double cValue = cExp.getValue();
			double rValue = 0.0;
			if (rMap.containsKey(exId)){
				rValue = rMap.get(exId);
			}
			double over = 0.0;
			if ((rValue >= cValue) && cValue < EX_TOP){
				over = (rValue - cValue) / (EX_TOP - cValue);
			}
			sum += Math.pow(over, 2);
			num ++;
		}
		double metric = 1.0;
		if (num != 0){
			metric = 1 - ((1/num) * Math.sqrt(sum)); 
		}
		return metric;
	}
	

	public ArrayList<Double> getMetric(String[] characterizationIds, String[] resourceIds) {
		ArrayList<Double> finalResult = new ArrayList<Double>();
		for (int i=0; i<resourceIds.length; i++){
			finalResult.add(getMetric(characterizationIds[i], resourceIds[i]));
		}
		return finalResult;
	}




}
