// By Mike
package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.PerformanceCharacterization;
import org.processmining.resourcerecommendation.models.PerformanceDimension;

public class PerformanceDimensionImpl implements PerformanceDimension {
	
	private String id;
	private Map<String, PerformanceCharacterization> characterizations;
	
	protected PerformanceDimensionImpl(){
		this.id = Dimensions.PERFORMANCE;
		this.characterizations = new LinkedHashMap<String, PerformanceCharacterization>();
	}
	
	public String getId(){
		return id;
	}
	
	public void addPerformance(String characterizationId, String resourceId, Double duration){
		PerformanceCharacterization characterization = characterizations.get(characterizationId);
		if (characterization == null){
			characterization = ResourceProcessFactory.createPerformanceCharacterization(characterizationId);
			this.characterizations.put(characterizationId, characterization);
		}
		characterization.addPerformance(resourceId, duration);	
	}
	
	public double getMetric(String characterizationId, String resourceId){
		if (!characterizations.containsKey(characterizationId)) return 0.0;
		long totalResource = characterizations.get(characterizationId).getTotal(resourceId);
		double max = characterizations.get(characterizationId).getMax();
		double min = characterizations.get(characterizationId).getMin();
		double average = characterizations.get(characterizationId).getAverage(resourceId);
		if(totalResource == 0) return 0.0;
		double result;
		if (min == max) result = 1.0;
		else result = (max - average) / (max-min);
		return result;
	}

	public ArrayList<Double> getMetric(String[] characterizationIds, String[] resourceIds) {
		ArrayList<Double> finalResult = new ArrayList<Double>();
		for (int i=0; i<resourceIds.length; i++){
			finalResult.add(getMetric(characterizationIds[i], resourceIds[i]));
		}
		return finalResult;
	}


}
