// By Mike
package org.processmining.resourcerecommendation.models.impl;

import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.QualityCharacterization;
import org.processmining.resourcerecommendation.models.QualityResource;


public class QualityCharacterizationImpl implements QualityCharacterization {
	
	private String characterizationId;
	private Map<String, QualityResource> resources;
	
	protected QualityCharacterizationImpl(String characterizationId){
		this.characterizationId = characterizationId;
		this.resources = new LinkedHashMap<String, QualityResource>();
	}
	
	public void addQuality(String resourceId, long quality){
		QualityResource resource = resources.get(resourceId);
		if (resource == null){
			resource = ResourceProcessFactory.createQualityResource(resourceId);
			resources.put(resourceId, resource);
		}
		resource.addQuality(quality);
	}
	
	public long getTotal(String resourceId) {
		long result = resources.containsKey(resourceId) ? 
				resources.get(resourceId).getTotal() : 0;
		return result;	
	}
	
	public long getTotal(){
		long result = 0;
		for (QualityResource resource: resources.values()){
			result += resource.getTotal();
		}
		return result;
	}

	public String getCharacterizationId() {
		return characterizationId;
	}

	public long getMin() {
		long min = Long.MAX_VALUE;
		for (QualityResource resource: resources.values()){
			if (resource.getTotal()> 0 && resource.getMin()< min) 
				min= resource.getMin(); 
		}
		return min;
	}
	
	public long getMax() {
		long max = Long.MIN_VALUE;
		for (QualityResource resource: resources.values()){
			if (resource.getTotal()> 0 && resource.getMax()> max) 
				max= resource.getMax(); 
		}
		return max;
	}

	public Double getAverage(String resourceId) {
		Double result = resources.containsKey(resourceId) ? 
				resources.get(resourceId).getAverage() : 0.0;
		return result;
	}
}
