// By Mike
package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.QualityCharacterization;
import org.processmining.resourcerecommendation.models.QualityDimension;

public class QualityDimensionImpl implements QualityDimension {
	
	private String id;
	private Map<String, QualityCharacterization> characterizations;
	
	protected QualityDimensionImpl(){
		this.id = Dimensions.QUALITY;
		this.characterizations = new LinkedHashMap<String, QualityCharacterization>();
	}
	
	public String getId(){
		return id;
	}
	
	public void addQuality(String characterizationId, String resourceId, long quality){
		QualityCharacterization characterization = characterizations.get(characterizationId);
		if (characterization == null){
			characterization = ResourceProcessFactory.createQualityCharacterization(characterizationId);
			this.characterizations.put(characterizationId, characterization);
		}
		characterization.addQuality(resourceId, quality);	
	}
	
	public double getMetric(String characterizationId, String resourceId){
		if (!characterizations.containsKey(characterizationId)) return 0.0;
		long totalResource = characterizations.get(characterizationId).getTotal(resourceId);
		long max = characterizations.get(characterizationId).getMax();
		long min = characterizations.get(characterizationId).getMin();
		double average = characterizations.get(characterizationId).getAverage(resourceId);
		if(totalResource == 0) return 0.0;
		double result;
		if (min == max) result = 1;
		else result = (average- min) / (max-min);
		return result;
	}

	public ArrayList<Double> getMetric(String[] characterizationIds, String[] resourceIds) {
		ArrayList<Double> finalResult = new ArrayList<Double>();
		for (int i=0; i<resourceIds.length; i++){
			finalResult.add(getMetric(characterizationIds[i], resourceIds[i]));
		}
		return finalResult;
	}


}
