package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.CostDimension;
import org.processmining.resourcerecommendation.models.Dimension;
import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.Expertise;
import org.processmining.resourcerecommendation.models.Factor;
import org.processmining.resourcerecommendation.models.FrequencyDimension;
import org.processmining.resourcerecommendation.models.OverQualificationDimension;
import org.processmining.resourcerecommendation.models.PerformanceDimension;
import org.processmining.resourcerecommendation.models.QualityDimension;
import org.processmining.resourcerecommendation.models.Resource;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.Typology;
import org.processmining.resourcerecommendation.models.UnderQualificationDimension;
import org.processmining.resourcerecommendation.models.Unit;
import org.processmining.resourcerecommendation.models.WorkloadDimension;

public class ResourceKnowledgeImpl implements ResourceKnowledge{
	
	private Map<String, Resource> resources;
	private Map<String, Unit> units;
	private Map<String, Factor> factors;
	private Map<String, Typology> typologies;
	private Map<String, Characterization> characterization;
	private Map<String, Expertise> expertises;
	
	private Map<String, Dimension> dimensions;
	
	
	protected ResourceKnowledgeImpl(){
		this.resources = new LinkedHashMap<String,Resource>();
		this.units = new LinkedHashMap<String, Unit>();
		this.factors = new LinkedHashMap<String, Factor>();
		this.typologies = new LinkedHashMap<String, Typology>();
		this.characterization = new LinkedHashMap<String, Characterization>();
		this.expertises = new LinkedHashMap<String, Expertise>();
		
		this.dimensions = new LinkedHashMap<String, Dimension>();
		this.dimensions.put(Dimensions.FREQUENCY, ResourceProcessFactory.createFrequencyDimension());
		this.dimensions.put(Dimensions.QUALITY, ResourceProcessFactory.createQualityDimension());
		this.dimensions.put(Dimensions.PERFORMANCE, ResourceProcessFactory.createPerformanceDimension());
		this.dimensions.put(Dimensions.COST, ResourceProcessFactory.createCostDimension());
		this.dimensions.put(Dimensions.UNDERQUALIFICATION, ResourceProcessFactory.createUnderQualificationDimension());
		this.dimensions.put(Dimensions.OVERQUALIFICATION, ResourceProcessFactory.createOverQualificationDimension());
		this.dimensions.put(Dimensions.WORKLOAD, ResourceProcessFactory.createWorkloadDimension());
	}


	public void addResource(Resource resource) {
		this.resources.put(resource.getId(), resource);
	}
	
	public List<Resource> getResources() {
		return new ArrayList<Resource>(this.resources.values());
	}
	
	public void addUnit(Unit unit){
		this.units.put(unit.getId(), unit);
	}
	
	public List<Unit> getUnits() {
		return new ArrayList<Unit>(this.units.values());
	}
	
	public void addFactor(Factor factor){
		this.factors.put(factor.getId(), factor);
	}
	
	public void addTypology(Typology typology){
		this.typologies.put(typology.getId(), typology);
	}
	
	public void addCharacterization(Characterization characterization){
		this.characterization.put(characterization.getId(), characterization);
	}
	
	public List<Characterization> getCharacterizations(){
		return new ArrayList<Characterization>(this.characterization.values());
	}
	
	public void addExpertise(Expertise expertise){
		this.expertises.put(expertise.getId(), expertise);
	}
	
	public List<Expertise> getExpertises() {
		return new ArrayList<Expertise>(this.expertises.values());
	}
	
	public List<Dimension> getDimensions(){
		return new ArrayList<Dimension>(dimensions.values());
	}
	
	public void addFrequency(String characterizationId, String resourceId){
		FrequencyDimension frequency = (FrequencyDimension) dimensions.get(Dimensions.FREQUENCY);
		frequency.addFrequency(characterizationId, resourceId);
	}

	public void addQuality(String characterizationId, String resourceId, long quality){
		QualityDimension qualityDimension = (QualityDimension) dimensions.get(Dimensions.QUALITY);
		qualityDimension.addQuality(characterizationId, resourceId, quality);
	}

	public void addPerformance(String characterizationId, String resourceId, Double duration){
		PerformanceDimension performanceDimension = (PerformanceDimension) dimensions.get(Dimensions.PERFORMANCE);
		performanceDimension.addPerformance(characterizationId, resourceId, duration);
	}
	
	public void addCost(String characterizationId, String resourceId, Double cost){
		CostDimension costDimension = (CostDimension) dimensions.get(Dimensions.COST);
		costDimension.addCost(characterizationId, resourceId, cost);
	}
	
	public void addRequiredExpertise(String characterizationId, String expertiseId, Long value){
		OverQualificationDimension overQualificationDimension = (OverQualificationDimension) dimensions.get(Dimensions.OVERQUALIFICATION);
		overQualificationDimension.addRequiredExpertise(characterizationId, expertiseId, value);
		UnderQualificationDimension underQualificationDimension = (UnderQualificationDimension) dimensions.get(Dimensions.UNDERQUALIFICATION);
		underQualificationDimension.addRequiredExpertise(characterizationId, expertiseId, value);
	}
	
	public void addResourceExpertise(String resourceId, String expertiseId, Long value){
		OverQualificationDimension overQualificationDimension = (OverQualificationDimension) dimensions.get(Dimensions.OVERQUALIFICATION);
		overQualificationDimension.addResourceExpertise(resourceId, expertiseId, value);
		UnderQualificationDimension underQualificationDimension = (UnderQualificationDimension) dimensions.get(Dimensions.UNDERQUALIFICATION);
		underQualificationDimension.addResourceExpertise(resourceId, expertiseId, value);
	}

	public void addWorkload(String resourceId, long max, long current){
		WorkloadDimension workloadDimension = (WorkloadDimension) dimensions.get(Dimensions.WORKLOAD);
		workloadDimension.addWorkload(resourceId, max, current);
	}
	
	public double getMetric(String dimensionId, String characterizationId, String resourceId) {
		Dimension dimension =  dimensions.get(dimensionId);
		return dimension.getMetric(characterizationId, resourceId);	
	}

	public ArrayList<Double> getMetric(String dimensionId, String[] characterizationIds, String[] resourceIds) {
		Dimension dimension =  dimensions.get(dimensionId);
		return dimension.getMetric(characterizationIds, resourceIds);	
	}

}
