package org.processmining.resourcerecommendation.models.impl;

import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.CostCharacterization;
import org.processmining.resourcerecommendation.models.CostDimension;
import org.processmining.resourcerecommendation.models.CostResource;
import org.processmining.resourcerecommendation.models.Expertise;
import org.processmining.resourcerecommendation.models.Factor;
import org.processmining.resourcerecommendation.models.FactorValue;
import org.processmining.resourcerecommendation.models.FrequencyCharacterization;
import org.processmining.resourcerecommendation.models.FrequencyDimension;
import org.processmining.resourcerecommendation.models.FrequencyResource;
import org.processmining.resourcerecommendation.models.OverQualificationDimension;
import org.processmining.resourcerecommendation.models.PerformanceCharacterization;
import org.processmining.resourcerecommendation.models.PerformanceDimension;
import org.processmining.resourcerecommendation.models.PerformanceResource;
import org.processmining.resourcerecommendation.models.QualityCharacterization;
import org.processmining.resourcerecommendation.models.QualityDimension;
import org.processmining.resourcerecommendation.models.QualityResource;
import org.processmining.resourcerecommendation.models.RecommendationElement;
import org.processmining.resourcerecommendation.models.Request;
import org.processmining.resourcerecommendation.models.Resource;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.ResourceRecommendation;
import org.processmining.resourcerecommendation.models.Typology;
import org.processmining.resourcerecommendation.models.UnderQualificationDimension;
import org.processmining.resourcerecommendation.models.Unit;
import org.processmining.resourcerecommendation.models.WorkloadDimension;
import org.processmining.resourcerecommendation.models.WorkloadResource;

public class ResourceProcessFactory {
	
	public static ResourceKnowledge createResourceKnowledge(){
		return new ResourceKnowledgeImpl();
	}

	public static Resource createResource(String id, String description) {
		return new ResourceImpl(id, description);
	}
	
	public static Unit createUnit(String id, String description) {
		return new UnitImpl(id, description);
	}
	
	public static Factor createFactor(String id, String description) {
		return new FactorImpl(id, description);
	}
	
	public static FactorValue createFactorValue(String id, String description) {
		return new FactorValueImpl(id, description);
	}
	
	public static Typology createTypology(String id, String description) {
		return new TypologyImpl(id, description);
	}
	
	public static Characterization createCharacterization(String id, String description, String unitId, String typologyId) {
		return new CharacterizationImpl(id, description, unitId, typologyId);
	}
	
	public static Expertise createExpertise(String id, String description) {
		return new ExpertiseImpl(id, description);
	}
	
	public static Request createRequest(String id, String description, String characterizationId){
		return new RequestImpl(id, description, characterizationId);
	}
	
	public static ResourceRecommendation createResourceRecommendation(){
		return new ResourceRecommendationImpl();
	}
	
	public static RecommendationElement createRecommendationElement(String resourceId, String resourceDescription){
		return new RecommendationElementImpl(resourceId, resourceDescription);
	}
	
	//Frequency
	public static FrequencyDimension createFrequencyDimension(){
		return new FrequencyDimensionImpl();
	}
	
	public static FrequencyCharacterization createFrequencyCharacterization(String characterizationId){
		return new FrequencyCharacterizationImpl(characterizationId);
	}
	
	public static FrequencyResource createFrequencyResource(String resourceId){
		return new FrequencyResourceImpl(resourceId);
	}
	
	//Quality
	public static QualityDimension createQualityDimension(){
		return new QualityDimensionImpl();
	}
	
	public static QualityCharacterization createQualityCharacterization(String characterizationId){
		return new QualityCharacterizationImpl(characterizationId);
	}
	
	public static QualityResource createQualityResource(String resourceId){
		return new QualityResourceImpl(resourceId);
	}
	
	//Performance
	public static PerformanceDimension createPerformanceDimension(){
		return new PerformanceDimensionImpl();
	}
	
	public static PerformanceCharacterization createPerformanceCharacterization(String characterizationId){
		return new PerformanceCharacterizationImpl(characterizationId);
	}
	
	public static PerformanceResource createPerformanceResource(String resourceId){
		return new PerformanceResourceImpl(resourceId);
	}
			
	//Cost
	public static CostDimension createCostDimension(){
		return new CostDimensionImpl();
	}
	
	public static CostCharacterization createCostCharacterization(String characterizationId){
		return new CostCharacterizationImpl(characterizationId);
	}
	
	public static CostResource createCostResource(String resourceId){
		return new CostResourceImpl(resourceId);
	}
	
	//UnderQualification
	public static UnderQualificationDimension createUnderQualificationDimension(){
		return new UnderQualificationDimensionImpl();
	}
	
	
	//OverQualification
	public static OverQualificationDimension createOverQualificationDimension(){
		return new OverQualificationDimensionImpl();
	}
	
	
	//Workload
	public static WorkloadDimension createWorkloadDimension(){
		return new WorkloadDimensionImpl();
	}
	
	public static WorkloadResource createWorkloadResource(String resourceId){
		return new WorkloadResourceImpl(resourceId);
	}
}
