package org.processmining.resourcerecommendation.models.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import org.processmining.resourcerecommendation.models.Dimensions;
import org.processmining.resourcerecommendation.models.WorkloadDimension;
import org.processmining.resourcerecommendation.models.WorkloadResource;

public class WorkloadDimensionImpl implements WorkloadDimension {
	
	private String id;
	private Map<String, WorkloadResource> resources;
	
	protected WorkloadDimensionImpl(){
		this.id = Dimensions.WORKLOAD;
		this.resources = new LinkedHashMap<String, WorkloadResource>();
	}
	public String getId(){
		return id;
	}
	
	public void addWorkload(String resourceId, long max, long current){
		WorkloadResource resource = resources.get(resourceId);
		if (resource == null){
			resource = ResourceProcessFactory.createWorkloadResource(resourceId);
			this.resources.put(resourceId, resource);
		}
		resource.addWorkload(max, current);	
	}
	
	public double getMetric(String resourceId){
		if (!resources.containsKey(resourceId)) return -10.0;
		int top = resources.get(resourceId).getTop();
		int current = resources.get(resourceId).getCurrent();
		int bottom = resources.get(resourceId).getBottom();
		
		double result;
		if (top == current)
			// It is not feasible to perform all required activities
			result = -10.0;
		else
			result = (double)(top-current)/ (top-bottom);
		
		return result;
	}

	public ArrayList<Double> getMetric(String[] resourceIds) {
		ArrayList<Double> finalResult = new ArrayList<Double>();
		
		for (int i=0; i<resourceIds.length; i++)
			if (!resources.containsKey(resourceIds[i]))
				finalResult.add(0.0);
			else
				finalResult.add(-10.0);
		
		for (int i=0; i<resourceIds.length; i++){
			if (finalResult.get(i) >= 0.0) break;
			
			int nOcurrences = 1;
			for (int j=i+1; j<resourceIds.length; j++)
				if (resourceIds[j] == resourceIds[i])
					nOcurrences++;
			
			if (nOcurrences == 1)
				finalResult.set(i, getMetric(resourceIds[i]));
			else {
				double value;

				int top = resources.get(resourceIds[i]).getTop();
				int current = resources.get(resourceIds[i]).getCurrent();
				int bottom = resources.get(resourceIds[i]).getBottom();

				double sumValues = 0.0;
				for (int k=0; k<nOcurrences; k++) {
					if (top == current) {
						// It is not feasible to perform all required activities
						sumValues = -10.0;
					} else {
						sumValues += (double)(top-current)/ (top-bottom);
						current++;
					}
				}
				if (sumValues == -10.0)
					value = -10.0;
				else
					value = sumValues / nOcurrences;
				
				finalResult.set(i, value);
				for (int j=i+1; j<resourceIds.length; j++)
					if (resourceIds[j] == resourceIds[i])
						finalResult.set(j, value);
			}
		}
		return finalResult;
	}

	public double getMetric(String characterizationId, String resourceId) {
		return getMetric(resourceId);
	}
	
	public ArrayList<Double> getMetric(String[] characterizationIds, String[] resourceIds) {
		return getMetric(resourceIds);
	}

	public int getTop(String resourceId) {
		if (!resources.containsKey(resourceId)) return 0;
		else return resources.get(resourceId).getTop();
	}

	public int getCurrent(String resourceId) {
		if (!resources.containsKey(resourceId)) return 0;
		else return resources.get(resourceId).getCurrent();
	}
}
