package org.processmining.resourcerecommendation.parameters;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.Dimension;
import org.processmining.resourcerecommendation.models.Request;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.impl.ResourceProcessFactory;

public class RecommendResourcesParameters extends PluginParametersImpl {
	
	public enum ALGORITHM {RECOMENDATION, ALLOCATION};
	
	private ALGORITHM algorithm;
	private int numRecommendations;
	private Map<String, Double> weights;
	private Map<String, Request> requests;
	
	
	public RecommendResourcesParameters(ResourceKnowledge knowledge){
		super();
		this.setTryConnections(false); //To avoid by default losing time with connections
		weights = new LinkedHashMap<String, Double>();
		requests = new LinkedHashMap<String, Request>();
		
		//DEFAULT
		algorithm = ALGORITHM.RECOMENDATION;
		numRecommendations = 3;
		
			//Default: a single request of the first characterization (arbitrary)
		String uuid = UUID.randomUUID().toString();
		Characterization characterization = knowledge.getCharacterizations().iterator().next();
		Request request = ResourceProcessFactory.createRequest("REQ:"+uuid, "REQ:"+uuid, characterization.getId());
		requests.put(request.getId(), request);
		
			//Default: all weights to 0.5
		List<Dimension> dimensions = knowledge.getDimensions();
		for (Dimension dimension: dimensions){
			weights.put(dimension.getId(), 0.5);
		}
	}
	
	public RecommendResourcesParameters(RecommendResourcesParameters parameters){
		super();
		this.setTryConnections(false); //To avoid by default losing time with connections
		this.algorithm = parameters.getAlgorithm();
		this.numRecommendations = parameters.getNumRecommendations();
		this.weights = new LinkedHashMap<String,Double>(parameters.weights);
		this.requests = new LinkedHashMap<String,Request>(parameters.requests);
	}
	
	public int getNumRecommendations() {
		return numRecommendations;
	}

	public Map<String, Double> getWeights() {
		return new LinkedHashMap<String,Double>(weights);
	}

	public List<Request> getRequests() {
		return new ArrayList<Request>(requests.values());
	}

	public void setNumRecommendations(int numRecommendations) {
		this.numRecommendations = numRecommendations;
	}

	public void setWeight(String dimensionId, Double weight) {
		this.weights.put(dimensionId, weight);
	}

	public void setWeights(Map<String, Double> weights) {
		this.weights = weights;
	}

	public void setRequests(Map<String, Request> requests) {
		this.requests = new LinkedHashMap<String, Request>(requests);
	}

	public ALGORITHM getAlgorithm() {
		return algorithm;
	}

	public void setAlgorithm(ALGORITHM algorithm) {
		this.algorithm = algorithm;
	}
	
	public int getNumRequests(String characterizationId){
		int num = 0;
		for (Request request: this.requests.values()){
			if (request.getCharacterizationId() == characterizationId){
				num ++;
			}
		}	
		return num;
	}
	

}
