// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmining.resourcerecommendation.plugins;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.resourcerecommendation.models.Characterization;
import org.processmining.resourcerecommendation.models.Dimension;
import org.processmining.resourcerecommendation.models.Resource;
import org.processmining.resourcerecommendation.models.ResourceKnowledge;
import org.processmining.resourcerecommendation.models.Unit;

import com.fluxicon.slickerbox.components.StackedCardsTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

@Plugin(name = "Visualize Resource Knowledge", 
		returnLabels = { "Visualized Resource Knowledge" }, 
		returnTypes = {JComponent.class }, 
		parameterLabels = { "Resource Knowledge" }, 
		userAccessible = true)
@Visualizer
public class VisualizeResourceKnowledgePlugin {

	@PluginVariant(requiredParameterLabels = { 0 })
	public static JComponent visualize(UIPluginContext context, ResourceKnowledge knowledge) {
		return visualize(knowledge);
	}

	public static JComponent visualize(ResourceKnowledge knowledge) {
		StackedCardsTabbedPane tabbedPane = SlickerFactory.instance().createStackedCardsPane();
		tabbedPane.addTab("General", createGeneral(knowledge));
		for(Dimension dimension: knowledge.getDimensions()){
			tabbedPane.addTab(dimension.getId(), createDimension(knowledge, dimension.getId()));
		}
		
		return tabbedPane;
	}
	
	public static JComponent createGeneral(ResourceKnowledge knowledge){
		StackedCardsTabbedPane pane = SlickerFactory.instance().createStackedCardsPane();

		//Resources
		JPanel resourcePanel = SlickerFactory.instance().createGradientPanel();
		List<Resource> resources = knowledge.getResources();
		double resourceSize[][] = computeTableLayoutPreferredSize(1, resources.size(), TableLayoutConstants.PREFERRED);
		resourcePanel.setLayout(new TableLayout(resourceSize));
		for (int i=0; i<resources.size(); i++){
			String id = resources.get(i).getId();
			String description = resources.get(i).getDescription();
			String string = id+": \t"+description;
			JLabel label = SlickerFactory.instance().createLabel(string);
			resourcePanel.add(label, "0 "+i);
		}
		pane.addTab("Resources", resourcePanel);
		
		//Units
		JPanel unitPanel = SlickerFactory.instance().createGradientPanel();
		List<Unit> units = knowledge.getUnits();
		double unitSize[][] = computeTableLayoutPreferredSize(1, units.size(), TableLayoutConstants.PREFERRED);
		unitPanel.setLayout(new TableLayout(unitSize));
		for (int i=0; i<units.size(); i++){
			String id = units.get(i).getId();
			String description = units.get(i).getDescription();
			String string = id+":\t"+description;
			JLabel label = SlickerFactory.instance().createLabel(string);
			unitPanel.add(label, "0 "+i);
		}
		pane.addTab("Units", unitPanel);
		
		
		return pane;
	}
	
	public static JComponent createDimension(ResourceKnowledge knowledge, String dimensionId){
		JPanel panel = SlickerFactory.instance().createGradientPanel();
		List<Resource> resources = knowledge.getResources();
		List<Characterization> characterizations = knowledge.getCharacterizations();
		double size[][] = computeTableLayoutPreferredSize(characterizations.size()+1, resources.size()+1, TableLayoutConstants.PREFERRED);
		panel.setLayout(new TableLayout(size));
		for(int r=0; r<resources.size()+1; r++){
			for(int c=0; c < characterizations.size()+1; c++){
				
				if (r == 0 && c == 0){
					//
				}
				else if (r == 0){
					JLabel label = SlickerFactory.instance().createLabel(" "+ characterizations.get(c-1).getId() +" ");
					panel.add(label, c+" "+r);
				}
				else if (c == 0){
					JLabel label = SlickerFactory.instance().createLabel(" "+resources.get(r-1).getId() +" ");
					panel.add(label, c+" "+r);
				}
				else{
					double metric = knowledge.getMetric(dimensionId, characterizations.get(c-1).getId(), resources.get(r-1).getId());
					JLabel label = SlickerFactory.instance().createLabel(" "+String.format("%.2f", metric)+" ");
					panel.add(label, c+" "+r);
				}
				
				
			}
		}
		
		return panel;
	}
	
	private static double[][] computeTableLayoutPreferredSize(int col, int row, double fill){
		double[] cols = new double[col];
		for (int i=0; i<col; i++){
			cols[i] = fill;
		}
		double[] rows = new double[row];
		for (int i=0; i<row; i++){
			rows[i] = fill;
		}
		double size[][] = {cols, rows};
		return size;
	}
	
	
	
	
}
