// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmining.resourcerecommendation.plugins;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.resourcerecommendation.models.RecommendationElement;
import org.processmining.resourcerecommendation.models.Request;
import org.processmining.resourcerecommendation.models.ResourceRecommendation;

import com.fluxicon.slickerbox.components.StackedCardsTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

@Plugin(name = "Visualize Resource Recommendation", 
		returnLabels = { "Visualized Resource Recommendation" }, 
		returnTypes = {JComponent.class }, 
		parameterLabels = { "Resource Recommendation" }, 
		userAccessible = true)
@Visualizer
public class VisualizeResourceRecommendationPlugin {

	@PluginVariant(requiredParameterLabels = { 0 })
	public static JComponent visualize(UIPluginContext context, ResourceRecommendation recommendation) {
		return visualize(recommendation);
	}

	public static JComponent visualize(ResourceRecommendation recommendation) {
		StackedCardsTabbedPane tabbedPane = SlickerFactory.instance().createStackedCardsPane();
		Map<Request, List<RecommendationElement>> ranking = recommendation.getRankings();
		for (Entry<Request, List<RecommendationElement>> entry: ranking.entrySet()){
			tabbedPane.addTab(entry.getKey().getId()+":\t"+entry.getKey().getCharacterizationId(), createRequest(recommendation, entry.getKey()));
		}
		return tabbedPane;
	}

	
	public static JComponent createRequest(ResourceRecommendation recommendation, Request request){
		List<RecommendationElement> ranking = recommendation.getRankings().get(request);
		List<String> scoreKeys = new ArrayList<String>(ranking.get(0).getScores().keySet());
		int columns = scoreKeys.size() +3;
		int rows = ranking.size() + 1;
		
		JPanel panel = SlickerFactory.instance().createGradientPanel();
		double size[][] = computeTableLayoutPreferredSize(columns, rows, TableLayoutConstants.PREFERRED);
		panel.setLayout(new TableLayout(size));		
		
		JLabel posHead = SlickerFactory.instance().createLabel(" pos ");
		panel.add(posHead, 0+" "+0);
		JLabel resHead = SlickerFactory.instance().createLabel(" res ");
		panel.add(resHead, 1+" "+0);
		JLabel overallHead = SlickerFactory.instance().createLabel(" overall ");
		panel.add(overallHead, 2+" "+0);
		int colhead = 3;
		for (String dimensionId: scoreKeys){
			JLabel dimensionHead = SlickerFactory.instance().createLabel(" "+ dimensionId +" ");
			panel.add(dimensionHead, colhead+" "+0);
			colhead ++;
		}
		
		
		for (int e=0; e < ranking.size(); e++){
			JLabel pos = SlickerFactory.instance().createLabel(" "+ e +" -> ");
			panel.add(pos, 0+" "+(e+1));
			JLabel res = SlickerFactory.instance().createLabel(" "+ ranking.get(e).getResourceId() +" ");
			panel.add(res, 1+" "+(e+1));
			JLabel overall = SlickerFactory.instance().createLabel(" " +String.format("%.2f", ranking.get(e).getOverallScore()) +" ");
			panel.add(overall, 2+" "+(e+1));
			int col = 3;
			for (String dimensionId: scoreKeys){
				JLabel dimension = SlickerFactory.instance().createLabel(" "+ String.format("%.2f", ranking.get(e).getScore(dimensionId)) +" ");
				panel.add(dimension, col+" "+(e+1));
				col ++;
			}
			
		}
		
		return panel;
	}
	
	private static double[][] computeTableLayoutPreferredSize(int col, int row, double fill){
		double[] cols = new double[col];
		for (int i=0; i<col; i++){
			cols[i] = fill;
		}
		double[] rows = new double[row];
		for (int i=0; i<row; i++){
			rows[i] = fill;
		}
		double size[][] = {cols, rows};
		return size;
	}
	
	
	
	
}
